"""
Algorithmic intervensions for satisfying fairness criteria.

There are three families of techniques:

1. **Pre-processing** - Adjust features in the dataset.
2. **In-processing** - Adjust the learning algorithm.
3. **Post-processing** - Adjust the learned classifier.

"""
