TOLGA_GENDER_ANALOGIES = [('she', 'he'),
                          ('herself', 'himself'),
                          ('her', 'his'),
                          ('woman', 'man'),
                          ('daughter', 'son'),
                          ('businesswoman', 'businessman'),
                          ('girl', 'boy'),
                          ('actress', 'actor'),
                          ('chairwoman', 'chairman'),
                          ('heroine', 'hero'),
                          ('mother', 'father'),
                          ('spokeswoman', 'spokesman'),
                          ('sister', 'brother'),
                          ('girls', 'boys'),
                          ('sisters', 'brothers'),
                          ('queen', 'king'),
                          ('niece', 'nephew'),
                          ('councilwoman', 'councilman'),
                          ('motherhood', 'fatherhood'),
                          ('women', 'men'),
                          ('petite', 'lanky'),
                          ('ovarian_cancer', 'prostate_cancer'),
                          ('Anne', 'John'),
                          ('schoolgirl', 'schoolboy'),
                          ('granddaughter', 'grandson'),
                          ('aunt', 'uncle'),
                          ('matriarch', 'patriarch'),
                          ('twin_sister', 'twin_brother'),
                          ('mom', 'dad'),
                          ('lesbian', 'gay'),
                          ('husband', 'younger_brother'),
                          ('gal', 'dude'),
                          ('lady', 'gentleman'),
                          ('sorority', 'fraternity'),
                          ('mothers', 'fathers'),
                          ('grandmother', 'grandfather'),
                          ('blouse', 'shirt'),
                          ('soprano', 'baritone'),
                          ('queens', 'kings'),
                          ('Jill', 'Greg'),
                          ('daughters', 'sons'),
                          ('grandma', 'grandpa'),
                          ('volleyball', 'football'),
                          ('diva', 'superstar'),
                          ('mommy', 'kid'),
                          ('Sarah', 'Matthew'),
                          ('hairdresser', 'barber'),
                          ('softball', 'baseball'),
                          ('goddess', 'god'),
                          ('Aisha', 'Jamal'),
                          ('waitress', 'waiter'),
                          ('princess', 'prince'),
                          ('filly', 'colt'),
                          ('mare', 'gelding'),
                          ('ladies', 'gentlemen'),
                          ('childhood', 'boyhood'),
                          ('interior_designer', 'architect'),
                          ('nun', 'priest'),
                          ('wig', 'beard'),
                          ('granddaughters', 'grandsons'),
                          ('girlfriends', 'buddies'),
                          ('gals', 'dudes'),
                          ('aunts', 'uncles'),
                          ('congresswoman', 'congressman'),
                          ('feminism', 'conservatism'),
                          ('bitch', 'bastard'),
                          ('hers', 'yours'),
                          ('bra', 'pants'),
                          ('moms', 'dads'),
                          ('nurse', 'surgeon'),
                          ('heiress', 'magnate'),
                          ('feminine', 'manly'),
                          ('glamorous', 'flashy'),
                          ('actresses', 'actors'),
                          ('registered_nurse', 'physician'),
                          ('cupcakes', 'pizzas'),
                          ('blond', 'burly'),
                          ('babe', 'fella'),
                          ('mums', 'blokes'),
                          ('gorgeous', 'magnificent'),
                          ('compatriot', 'countryman'),
                          ('fabulous', 'terrific'),
                          ('breast', 'prostate'),
                          ('starlet', 'youngster'),
                          ('Laurie', 'Brett'),
                          ('kids', 'guys'),
                          ('sewing', 'carpentry'),
                          ('kinda', 'guy'),
                          ('headscarf', 'turban'),
                          ('siblings', 'elder_brother'),
                          ('charming', 'affable'),
                          ('sassy', 'snappy'),
                          ('cosmetics', 'pharmaceuticals'),
                          ('estrogen', 'testosterone'),
                          ('handbag', 'briefcase'),
                          ('housewife', 'shopkeeper'),
                          ('fillies', 'colts'),
                          ('nieces', 'nephews'),
                          ('whore', 'coward'),
                          ('boyfriend', 'pal'),
                          ('salon', 'barbershop'),
                          ('Latonya', 'Leroy'),
                          ('vagina', 'penis'),
                          ('breast_cancer', 'lymphoma'),
                          ('vocalist', 'guitarist'),
                          ('me', 'him'),
                          ('children', 'youngsters'),
                          ('adorable', 'goofy'),
                          ('giggling', 'grinning'),
                          ('cheerful', 'jovial'),
                          ('lovely', 'brilliant'),
                          ('giggle', 'chuckle'),
                          ('bras', 'trousers'),
                          ('wedding_dress', 'tuxedo'),
                          ('singer', 'frontman'),
                          ('netball', 'rugby'),
                          ('rebounder', 'playmaker'),
                          ('vocalists', 'trumpeter'),
                          ('nude', 'shirtless'),
                          ('beautiful', 'majestic'),
                          ('feisty', 'mild_mannered'),
                          ('feminists', 'socialists'),
                          ('nanny', 'chauffeur'),
                          ('females', 'males'),
                          ('pediatrician', 'orthopedic_surgeon'),
                          ('teenage_girls', 'youths'),
                          ('pink', 'red'),
                          ('convent', 'monastery'),
                          ('midwife', 'doctor'),
                          ('feminist', 'liberal'),
                          ('gown', 'blazer'),
                          ('blonde', 'blond'),
                          ('stepdaughter', 'stepson'),
                          ('wonderful', 'great'),
                          ('breasts', 'genitals'),
                          ('luscious', 'crisp'),
                          ('judgmental', 'arrogant'),
                          ('skirts', 'shorts'),
                          ('middle_aged', 'bearded'),
                          ('spokespeople', 'spokesmen'),
                          ('female', 'male'),
                          ('beauty', 'grandeur'),
                          ('salesperson', 'salesman'),
                          ('witch', 'demon'),
                          ('male_counterparts', 'counterparts'),
                          ('violinist', 'virtuoso'),
                          ('practicality', 'durability'),
                          ('boobs', 'ass'),
                          ('dolls', 'replicas'),
                          ('husbands', 'wives'),
                          ('ponytail', 'mustache'),
                          ('sexism', 'racism'),
                          ('mammogram', 'colonoscopy'),
                          ('sweater', 'jersey'),
                          ('hysterical', 'comical'),
                          ('uterus', 'intestine'),
                          ('rehearse', 'improvise'),
                          ('classmates', 'teammates'),
                          ('stroller', 'bicycle'),
                          ('presenter', 'broadcaster'),
                          ('friends', 'buddy'),
                          ('dresses', 'shirts'),
                          ('eating_disorders', 'alcoholism'),
                          ('kindness', 'humility'),
                          ('cute', 'clever'),
                          ('sobbed', 'grinned'),
                          ('baking', 'roasting'),
                          ('sweaters', 'jerseys'),
                          ('crafting', 'drafting'),
                          ('nuns', 'monk'),
                          ('lesbians', 'homosexual'),
                          ('guidance_counselor', 'headmaster'),
                          ('terrifying', 'fearsome'),
                          ('libero', 'midfielders'),
                          ('classmate', 'teammate'),
                          ('underclassmen', 'players'),
                          ('pianist', 'maestro'),
                          ('boyfriends', 'friends'),
                          ('teenager', 'lad'),
                          ('giggles', 'chuckles'),
                          ('gender', 'racial'),
                          ('hair_salon', 'pizzeria'),
                          ('pitcher', 'starter'),
                          ('lovers', 'aficionados'),
                          ('lingerie', 'menswear'),
                          ('kittens', 'neutered'),
                          ('latex', 'rubber'),
                          ('dress', 'garb'),
                          ('alluring', 'intriguing'),
                          ('buns', 'sausages'),
                          ('cosmetic_surgery', 'surgery'),
                          ('foal', 'stallion'),
                          ('sexy', 'nerdy'),
                          ('delightful', 'superb'),
                          ('cubs', 'lions'),
                          ('self_esteem', 'morale'),
                          ('rower', 'skipper'),
                          ('bingo', 'gambling'),
                          ('figure_skating', 'hockey'),
                          ('stepmother', 'eldest_son'),
                          ('swimwear', 'sportswear'),
                          ('teen', 'youth'),
                          ('gymnastics', 'weightlifting'),
                          ('tissues', 'cartilage'),
                          ('enchanting', 'splendid'),
                          ('hooker', 'winger'),
                          ('graphic_designer', 'carpenter'),
                          ('layups', 'downfield'),
                          ('sophomores', 'rookies'),
                          ('singers', 'guitarists'),
                          ('backcourt', 'playmakers'),
                          ('hugs', 'shook_hands'),
                          ('sexist', 'racist'),
                          ('cheesecake', 'pizza'),
                          ('sultry', 'mellow'),
                          ('silicone', 'polymer'),
                          ('gymnasts', 'athletes'),
                          ('ribbons', 'flags'),
                          ('captivating', 'electrifying'),
                          ('gymnast', 'athlete'),
                          ('cried', 'chuckled'),
                          ('ethereal', 'brooding'),
                          ('cats', 'pigeons'),
                          ('implants', 'stents'),
                          ('cougar', 'wolves'),
                          ('fiance', 'cousin'),
                          ('cheery', 'amiable'),
                          ('servicemen', 'veterans'),
                          ('hair', 'facial_hair'),
                          ('sophomore', 'pounder'),
                          ('midwives', 'doctors'),
                          ('chairperson', 'managing_director'),
                          ('meter_hurdles', 'yard_dash'),
                          ('scream', 'boo'),
                          ('glam', 'retro'),
                          ('terrified', 'mad'),
                          ('shrill', 'louder'),
                          ('lupus', 'multiple_myeloma'),
                          ('uptight', 'cocky'),
                          ('hubby', 'pals'),
                          ('exclaimed', 'quipped'),
                          ('choreography', 'footwork'),
                          ('veil', 'cloak'),
                          ('panties', 'socks'),
                          ('gowns', 'robes'),
                          ('designers', 'architects'),
                          ('foul_trouble', 'defensively'),
                          ('amazing', 'unbelievable'),
                          ('artistry', 'genius'),
                          ('sensual', 'moody'),
                          ('campers', 'camp'),
                          ('Allison', 'Todd'),
                          ('freaking', 'heck'),
                          ('intuition', 'gut_feeling'),
                          ('eating_disorder', 'addiction'),
                          ('frontcourt', 'swingman'),
                          ('hostess', 'bartender'),
                          ('child_endangerment', 'aggravated_assault'),
                          ('masculine', 'macho'),
                          ('novelist', 'philosopher'),
                          ('putback', 'yarder'),
                          ('ex_boyfriend', 'ex_girlfriend'),
                          ('incredibly', 'obviously'),
                          ('beautify', 'rehabilitate'),
                          ('designed', 'engineered'),
                          ('mares', 'thoroughbred'),
                          ('satin', 'leather'),
                          ('hurler', 'lefthander'),
                          ('bun', 'burger'),
                          ('heartbreaking', 'humbling'),
                          ('romantic_comedy', 'flick'),
                          ('fourteen', 'eleven'),
                          ('flight_attendant', 'pilots'),
                          ('cupcake', 'donut'),
                          ('glitter', 'confetti'),
                          ('sobbing', 'smiled'),
                          ('unassisted_goal', 'powerplay'),
                          ('memoir', 'autobiography'),
                          ('cigarette', 'cigar'),
                          ('animal_cruelty', 'dogfighting'),
                          ('baby', 'daddy'),
                          ('outfits', 'uniforms'),
                          ('bride', 'groom'),
                          ('sprinter', 'speedster'),
                          ('estranged_husband', 'stepfather'),
                          ('singer_songwriter', 'musician'),
                          ('entrepreneurs', 'businessmen'),
                          ('grad', 'alumnus'),
                          ('lover', 'enthusiast'),
                          ('celebrities', 'superstars'),
                          ('kissing', 'shaking_hands'),
                          ('auditions', 'tryout'),
                          ('backstretch', 'straightaway'),
                          ('pregnancy', 'gestation'),
                          ('librarian', 'curator'),
                          ('anime', 'videogames'),
                          ('heartbroken', 'dejected'),
                          ('cervical_cancer', 'pancreatic_cancer'),
                          ('regionals', 'playoffs'),
                          ('Carrie', 'Laurie'),
                          ('teenage_girl', 'teenager'),
                          ('coworkers', 'colleagues'),
                          ('sergeants', 'lieutenants'),
                          ('paralegal', 'accountant'),
                          ('stressful', 'frustrating'),
                          ('duets', 'saxophonist'),
                          ('plucky', 'hapless'),
                          ('realtor', 'builder'),
                          ('cocktails', 'beers'),
                          ('closets', 'lockers'),
                          ('doubleheader', 'game'),
                          ('thighs', 'backside'),
                          ('wellness', 'fitness'),
                          ('giant_slalom', 'skied'),
                          ('neurotic', 'eccentric'),
                          ('coordinator', 'manager'),
                          ('antiques', 'memorabilia'),
                          ('dancers', 'drummers'),
                          ('layup', 'touchdown_pass'),
                          ('manga', 'comic_books'),
                          ('horrified', 'disappointed'),
                          ('empowering', 'motivating'),
                          ('creepy', 'menacing'),
                          ('insecurities', 'frustrations'),
                          ('middle_blocker', 'leadoff_hitter'),
                          ('adored', 'revered'),
                          ('exercises', 'drills'),
                          ('skirted', 'evaded'),
                          ('jewelry', 'collectibles'),
                          ('choral', 'composer'),
                          ('confided', 'intimated'),
                          ('dermatologist', 'neurosurgeon'),
                          ('attractiveness', 'competitiveness'),
                          ('housekeeper', 'janitor'),
                          ('thyroid', 'inflammation'),
                          ('vocals', 'drummer'),
                          ('scarf', 'jacket'),
                          ('burlesque', 'rock_n_roll'),
                          ('athlete', 'player'),
                          ('chic', 'minimalist'),
                          ('cerebral_palsy', 'spinal_cord_injury'),
                          ('horrid', 'woeful'),
                          ('transgender', 'homophobic'),
                          ('clique', 'inner_circle'),
                          ('pampering', 'luxurious'),
                          ('associate_professor', 'professor_emeritus'),
                          ('sexuality', 'homosexuality'),
                          ('reunite', 'rejoin'),
                          ('caring', 'selfless'),
                          ('freshman', 'redshirt_freshman'),
                          ('plunging', 'slumping'),
                          ('spokesperson', 'statement'),
                          ('prostitution', 'drug_trafficking'),
                          ('characters', 'villain'),
                          ('therapist', 'neurologist'),
                          ('maids', 'servants'),
                          ('mama', 'ya'),
                          ('delightfully', 'brilliantly'),
                          ('boutiques', 'retail_outlets'),
                          ('hormonal', 'metabolism'),
                          ('puppy', 'bulldog'),
                          ('pinch_runner', 'pinch_hitter'),
                          ('behaviors', 'tendencies'),
                          ('appalled', 'displeased'),
                          ('forms', 'form'),
                          ('nonprofit', 'organization'),
                          ('tamoxifen', 'statins'),
                          ('mysterious', 'enigmatic'),
                          ('maternity', 'obstetrics'),
                          ('mentors', 'mentor'),
                          ('worker', 'foreman'),
                          ('anorexia', 'depression'),
                          ('critters', 'beasts'),
                          ('feline', 'rodent'),
                          ('stereotype', 'mentality'),
                          ('starvation', 'famine'),
                          ('fairy', 'magical'),
                          ('gossip', 'rumor_mill'),
                          ('confesses', 'concedes'),
                          ('activists', 'supporters'),
                          ('flower', 'ornamental'),
                          ('contests', 'games'),
                          ('procuring', 'acquiring'),
                          ('silver_medalist', 'compatriot'),
                          ('purse', 'wallet'),
                          ('nurses', 'physicians'),
                          ('sectionals', 'championship'),
                          ('refill', 'recharge'),
                          ('inadequate', 'ineffective'),
                          ('prettiest', 'finest'),
                          ('seatbelt', 'wearing_helmet'),
                          ('rehearsal', 'improvisation'),
                          ('interns', 'fellows'),
                          ('adds', 'added'),
                          ('harmonies', 'guitar'),
                          ('comfy', 'comfortable'),
                          ('menopause', 'heart_disease'),
                          ('scripture', 'disciples'),
                          ('cries', 'chants'),
                          ('experimenting', 'tinkering'),
                          ('underweight', 'underperform'),
                          ('holistic_approach', 'approach'),
                          ('costumes', 'props'),
                          ('curves', 'curve'),
                          ('bartender', 'bouncer'),
                          ('seductive', 'mesmerizing'),
                          ('clarinet', 'trombone'),
                          ('transformative', 'visionary'),
                          ('ok', 'alright'),
                          ('unusual', 'unorthodox'),
                          ('osteoporosis', 'atrial_fibrillation'),
                          ('screams', 'roar'),
                          ('incumbents', 'incumbent'),
                          ('baseman', 'offensive_lineman'),
                          ('strawberry', 'potato'),
                          ('steals', 'tackles'),
                          ('walker', 'crutches'),
                          ('cook', 'grill'),
                          ('workshop', 'meeting'),
                          ('elegance', 'style'),
                          ('emotionally', 'mentally'),
                          ('beloved', 'legendary'),
                          ('breast_milk', 'sperm'),
                          ('minivan', 'pickup'),
                          ('complainants', 'accusers'),
                          ('knitting', 'yarn'),
                          ('artisans', 'craftsmen'),
                          ('strollers', 'scooters'),
                          ('caseload', 'workload'),
                          ('smile', 'grin'),
                          ('helpful', 'useful'),
                          ('caretakers', 'caretaker'),
                          ('blokes', 'bloke'),
                          ('provocative', 'incendiary'),
                          ('ultrasound', 'x_ray'),
                          ('colorful', 'flamboyant'),
                          ('counselor', 'adviser'),
                          ('borrower', 'loan'),
                          ('brightness', 'sharpness'),
                          ('staffing', 'manpower'),
                          ('overwhelmed', 'frustrated'),
                          ('bowlers', 'seamer'),
                          ('bunny', 'monkey'),
                          ('devotion', 'allegiance'),
                          ('advocates', 'critics'),
                          ('brides', 'weddings'),
                          ('cigarettes', 'cigars'),
                          ('dancer', 'entertainer'),
                          ('relationships', 'rapport'),
                          ('attitudes', 'mindset'),
                          ('strives', 'wants'),
                          ('bearish', 'bullish'),
                          ('deadlines', 'deadline'),
                          ('proponents', 'detractors'),
                          ('pitchers', 'ballclub'),
                          ('carriages', 'locomotive'),
                          ('shortstop', 'fullback'),
                          ('exhibit', 'exhibition'),
                          ('credit_crunch', 'slump'),
                          ('watchers', 'pundits'),
                          ('hitter', 'southpaw'),
                          ('scary', 'dangerous'),
                          ('silver_medals', 'championships'),
                          ('expiration_date', 'expires'),
                          ('tenacious', 'hard_nosed'),
                          ('gosh', 'yeah'),
                          ('bright_colors', 'colors'),
                          ('retro', 'throwback'),
                          ('underpass', 'tunnel'),
                          ('thrilled', 'confident'),
                          ('warship', 'destroyer'),
                          ('resignations', 'resignation'),
                          ('reaffirmed', 'reiterated'),
                          ('technical_fouls', 'ejection'),
                          ('sheets', 'sheet'),
                          ('makeover', 'revamp'),
                          ('bookkeeper', 'treasurer'),
                          ('callous', 'foolish'),
                          ('dignified', 'honorable'),
                          ('elegantly', 'superbly'),
                          ('foods', 'foodstuffs'),
                          ('cashier', 'robber'),
                          ('assistant_professor', 'economics_professor'),
                          ('downsize', 'restructure'),
                          ('details', 'specifics'),
                          ('healthful', 'healthy'),
                          ('luminous', 'dazzling'),
                          ('charm', 'swagger'),
                          ('pastry', 'sausage'),
                          ('free_throws', 'recovered_fumble'),
                          ('naysayers', 'doubters'),
                          ('embryo', 'embryonic'),
                          ('stalker', 'pedophile'),
                          ('hormones', 'growth_hormone'),
                          ('irresponsibility', 'recklessness'),
                          ('intangible_assets', 'intangibles'),
                          ('descriptive', 'succinct'),
                          ('screamed', 'shouted'),
                          ('manipulative', 'devious'),
                          ('audio_replay', 'replay')]
